@@ -51,6 +51,13 @@ _service_field_map = {
 }
 
 
+def has_any_fields_set(message):
+    for field_desc, value in message.ListFields():
+        if value:
+            return True
+    return False
+
+
 async def call_and_log_error(co):
     """Call a GRPC coroutine and log any errors from it."""
     try:
@@ -109,7 +116,10 @@ async def _sync_objects(model, company_uuid, source, bss_objs, model_objs, creat
                                 if smval != sbval:
                                     achgs.append(f"{sf.name}[{smval}->{sbval}]")
                         elif bval and not mval:
-                            achgs = [f"null->{bval}"]
+                            # PPA-318 the model sets an empty attachment to no
+                            # attachment, so handle the comparison properly here.
+                            if has_any_fields_set(bval):
+                                achgs = [f"null->{bval}"]
                         elif mval and not bval:
                             achgs = [f"{bval}->null"]
                         if not achgs:
