@@ -6,7 +6,9 @@ from uuid import uuid4
 from preseem_protobuf.network_poller import network_poller_pb2
 from preseem import network_element_update
 
+
 class TestNetworkElementUpdate(unittest.TestCase):
+
     def setUp(self):
         pass
 
@@ -99,11 +101,10 @@ class TestNetworkElementUpdate(unittest.TestCase):
         ip_mac.port = None
         self.assertEqual(len(ip_mac.errctx.errors), 2)
         ip_mac.errctx = None
-        
+
         del neu.ip_macs[0]
         self.assertEqual(len(neu.ip_macs), 0)
 
-
         # Test Route
         self.assertEqual(list(neu.routes), [])
         route = neu.routes.add()
@@ -142,7 +143,6 @@ class TestNetworkElementUpdate(unittest.TestCase):
         del neu.routes[0]
         self.assertEqual(len(neu.routes), 0)
 
-
         # Test DhcpLease
         self.assertEqual(list(neu.dhcp_leases), [])
         dhcp_lease = neu.dhcp_leases.add()
@@ -197,7 +197,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         del neu.pppoe_clients[0]
         self.assertEqual(len(neu.pppoe_clients), 0)
-        
+
         self.assertEqual(list(pppoe_client.network_prefixes), [])
         pppoe_client.network_prefixes.append('192.0.2.100')
         self.assertEqual(len(pppoe_client.network_prefixes), 1)
@@ -210,7 +210,6 @@ class TestNetworkElementUpdate(unittest.TestCase):
         del pppoe_client.network_prefixes[0]
         self.assertEqual(len(pppoe_client.network_prefixes), 0)
 
-
         # Check copy behavior of sub-message assignment
         ne = network_element_update.NetworkElement()
         self.assertEqual(ne.management_ip, None)
@@ -229,7 +228,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         self.assertEqual(ne.manufacturer, None)
         ne.manufacturer = 'my-mfg'
-        self.assertEqual(ne.manufacturer , 'my-mfg')
+        self.assertEqual(ne.manufacturer, 'my-mfg')
         ne.manufacturer = None
         self.assertEqual(ne.manufacturer, None)
 
@@ -276,7 +275,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         ne.errctx = network_element_update.ErrorContext()
         self.assertEqual(ne.cpu, None)
-        ne.cpu= 5.5
+        ne.cpu = 5.5
         self.assertEqual(round(ne.cpu, 4), 5.5)
         ne.cpu = -1
         self.assertEqual(round(ne.cpu, 4), 5.5)
@@ -418,7 +417,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.in_octets, None)
-        intf.in_octets = pow(2,31) + 1
+        intf.in_octets = pow(2, 31) + 1
         self.assertEqual(intf.in_octets, pow(2, 31) + 1)
         intf.in_octets = -1
         self.assertEqual(intf.in_octets, pow(2, 31) + 1)
@@ -431,7 +430,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.in_ucast_pkts, None)
-        intf.in_ucast_pkts = pow(2,32) + 1
+        intf.in_ucast_pkts = pow(2, 32) + 1
         self.assertEqual(intf.in_ucast_pkts, pow(2, 32) + 1)
         intf.in_ucast_pkts = -1
         self.assertEqual(intf.in_ucast_pkts, pow(2, 32) + 1)
@@ -444,7 +443,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.in_nucast_pkts, None)
-        intf.in_nucast_pkts = pow(2,33) + 1
+        intf.in_nucast_pkts = pow(2, 33) + 1
         self.assertEqual(intf.in_nucast_pkts, pow(2, 33) + 1)
         intf.in_nucast_pkts = -1
         self.assertEqual(intf.in_nucast_pkts, pow(2, 33) + 1)
@@ -457,9 +456,9 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.in_discards, None)
-        intf.in_discards = pow(2,34) + 1
+        intf.in_discards = pow(2, 34) + 1
         self.assertEqual(intf.in_discards, pow(2, 34) + 1)
-        intf.in_discards= -1
+        intf.in_discards = -1
         self.assertEqual(intf.in_discards, pow(2, 34) + 1)
         intf.in_discards = pow(2, 63) + 1
         self.assertEqual(intf.in_discards, pow(2, 34) + 1)
@@ -470,7 +469,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.in_errors, None)
-        intf.in_errors = pow(2,35) + 1
+        intf.in_errors = pow(2, 35) + 1
         self.assertEqual(intf.in_errors, pow(2, 35) + 1)
         intf.in_errors = -1
         self.assertEqual(intf.in_errors, pow(2, 35) + 1)
@@ -483,7 +482,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.in_unknown_protos, None)
-        intf.in_unknown_protos = pow(2,36) + 1
+        intf.in_unknown_protos = pow(2, 36) + 1
         self.assertEqual(intf.in_unknown_protos, pow(2, 36) + 1)
         intf.in_unknown_protos = -1
         self.assertEqual(intf.in_unknown_protos, pow(2, 36) + 1)
@@ -496,7 +495,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.out_octets, None)
-        intf.out_octets = pow(2,37) + 1
+        intf.out_octets = pow(2, 37) + 1
         self.assertEqual(intf.out_octets, pow(2, 37) + 1)
         intf.out_octets = -1
         self.assertEqual(intf.out_octets, pow(2, 37) + 1)
@@ -509,7 +508,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.out_ucast_pkts, None)
-        intf.out_ucast_pkts = pow(2,38) + 1
+        intf.out_ucast_pkts = pow(2, 38) + 1
         self.assertEqual(intf.out_ucast_pkts, pow(2, 38) + 1)
         intf.out_ucast_pkts = -1
         self.assertEqual(intf.out_ucast_pkts, pow(2, 38) + 1)
@@ -522,7 +521,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.out_nucast_pkts, None)
-        intf.out_nucast_pkts = pow(2,39) + 1
+        intf.out_nucast_pkts = pow(2, 39) + 1
         self.assertEqual(intf.out_nucast_pkts, pow(2, 39) + 1)
         intf.out_nucast_pkts = -1
         self.assertEqual(intf.out_nucast_pkts, pow(2, 39) + 1)
@@ -535,7 +534,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.out_discards, None)
-        intf.out_discards = pow(2,40) + 1
+        intf.out_discards = pow(2, 40) + 1
         self.assertEqual(intf.out_discards, pow(2, 40) + 1)
         intf.out_discards = -1
         self.assertEqual(intf.out_discards, pow(2, 40) + 1)
@@ -548,7 +547,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         intf.errctx = network_element_update.ErrorContext()
         self.assertEqual(intf.out_errors, None)
-        intf.out_errors = pow(2,41) + 1
+        intf.out_errors = pow(2, 41) + 1
         self.assertEqual(intf.out_errors, pow(2, 41) + 1)
         intf.out_errors = -1
         self.assertEqual(intf.out_errors, pow(2, 41) + 1)
@@ -578,6 +577,18 @@ class TestNetworkElementUpdate(unittest.TestCase):
         intf.poller_hash = poller_hash
         self.assertEqual(poller_hash, intf.poller_hash)
 
+        self.assertEqual(intf.manufacturer, None)
+        intf.manufacturer = 'intf-mfg'
+        self.assertEqual(intf.manufacturer, 'intf-mfg')
+        intf.manufacturer = None
+        self.assertEqual(intf.manufacturer, None)
+
+        self.assertEqual(intf.model, None)
+        intf.model = 'intf-mod'
+        self.assertEqual(intf.model, 'intf-mod')
+        intf.model = None
+        self.assertEqual(intf.model, None)
+
         # Link
         link = intf.links.add()
         link.mac_address = '0a:00:00:0d:0e:0f'
@@ -585,7 +596,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.in_octets, None)
-        link.in_octets = pow(2,31) + 1
+        link.in_octets = pow(2, 31) + 1
         self.assertEqual(link.in_octets, pow(2, 31) + 1)
         link.in_octets = -1
         self.assertEqual(link.in_octets, pow(2, 31) + 1)
@@ -598,7 +609,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.in_ucast_pkts, None)
-        link.in_ucast_pkts = pow(2,32) + 1
+        link.in_ucast_pkts = pow(2, 32) + 1
         self.assertEqual(link.in_ucast_pkts, pow(2, 32) + 1)
         link.in_ucast_pkts = -1
         self.assertEqual(link.in_ucast_pkts, pow(2, 32) + 1)
@@ -611,7 +622,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.in_nucast_pkts, None)
-        link.in_nucast_pkts = pow(2,33) + 1
+        link.in_nucast_pkts = pow(2, 33) + 1
         self.assertEqual(link.in_nucast_pkts, pow(2, 33) + 1)
         link.in_nucast_pkts = -1
         self.assertEqual(link.in_nucast_pkts, pow(2, 33) + 1)
@@ -624,9 +635,9 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.in_discards, None)
-        link.in_discards = pow(2,34) + 1
+        link.in_discards = pow(2, 34) + 1
         self.assertEqual(link.in_discards, pow(2, 34) + 1)
-        link.in_discards= -1
+        link.in_discards = -1
         self.assertEqual(link.in_discards, pow(2, 34) + 1)
         link.in_discards = pow(2, 63) + 1
         self.assertEqual(link.in_discards, pow(2, 34) + 1)
@@ -637,7 +648,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.in_errors, None)
-        link.in_errors = pow(2,35) + 1
+        link.in_errors = pow(2, 35) + 1
         self.assertEqual(link.in_errors, pow(2, 35) + 1)
         link.in_errors = -1
         self.assertEqual(link.in_errors, pow(2, 35) + 1)
@@ -650,7 +661,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.in_unknown_protos, None)
-        link.in_unknown_protos = pow(2,36) + 1
+        link.in_unknown_protos = pow(2, 36) + 1
         self.assertEqual(link.in_unknown_protos, pow(2, 36) + 1)
         link.in_unknown_protos = -1
         self.assertEqual(link.in_unknown_protos, pow(2, 36) + 1)
@@ -663,7 +674,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.out_octets, None)
-        link.out_octets = pow(2,37) + 1
+        link.out_octets = pow(2, 37) + 1
         self.assertEqual(link.out_octets, pow(2, 37) + 1)
         link.out_octets = -1
         self.assertEqual(link.out_octets, pow(2, 37) + 1)
@@ -676,7 +687,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.out_ucast_pkts, None)
-        link.out_ucast_pkts = pow(2,38) + 1
+        link.out_ucast_pkts = pow(2, 38) + 1
         self.assertEqual(link.out_ucast_pkts, pow(2, 38) + 1)
         link.out_ucast_pkts = -1
         self.assertEqual(link.out_ucast_pkts, pow(2, 38) + 1)
@@ -689,7 +700,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.out_nucast_pkts, None)
-        link.out_nucast_pkts = pow(2,39) + 1
+        link.out_nucast_pkts = pow(2, 39) + 1
         self.assertEqual(link.out_nucast_pkts, pow(2, 39) + 1)
         link.out_nucast_pkts = -1
         self.assertEqual(link.out_nucast_pkts, pow(2, 39) + 1)
@@ -702,7 +713,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.out_discards, None)
-        link.out_discards = pow(2,40) + 1
+        link.out_discards = pow(2, 40) + 1
         self.assertEqual(link.out_discards, pow(2, 40) + 1)
         link.out_discards = -1
         self.assertEqual(link.out_discards, pow(2, 40) + 1)
@@ -715,7 +726,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.out_errors, None)
-        link.out_errors = pow(2,41) + 1
+        link.out_errors = pow(2, 41) + 1
         self.assertEqual(link.out_errors, pow(2, 41) + 1)
         link.out_errors = -1
         self.assertEqual(link.out_errors, pow(2, 41) + 1)
@@ -728,7 +739,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         link.errctx = network_element_update.ErrorContext()
         self.assertEqual(link.out_retransmits, None)
-        link.out_retransmits = pow(2,42) + 1
+        link.out_retransmits = pow(2, 42) + 1
         self.assertEqual(link.out_retransmits, pow(2, 42) + 1)
         link.out_retransmits = -1
         self.assertEqual(link.out_retransmits, pow(2, 42) + 1)
@@ -919,12 +930,14 @@ class TestNetworkElementUpdate(unittest.TestCase):
         radio.errctx = network_element_update.ErrorContext()
         self.assertEqual(radio.wifi_mode, None)
         radio.wifi_mode = network_poller_pb2.WifiStandard.WIFI_STANDARD_80211B
-        self.assertEqual(radio.wifi_mode, network_poller_pb2.WifiStandard.WIFI_STANDARD_80211B)
+        self.assertEqual(radio.wifi_mode,
+                         network_poller_pb2.WifiStandard.WIFI_STANDARD_80211B)
         radio.wifi_mode = network_poller_pb2.WifiStandard.WIFI_STANDARD_INVALID
         self.assertEqual(len(radio.errctx.errors), 1)
         self.assertEqual(radio.wifi_mode, None)
         radio.wifi_mode = network_poller_pb2.WifiStandard.WIFI_STANDARD_80211AY
-        self.assertEqual(radio.wifi_mode, network_poller_pb2.WifiStandard.WIFI_STANDARD_80211AY)
+        self.assertEqual(radio.wifi_mode,
+                         network_poller_pb2.WifiStandard.WIFI_STANDARD_80211AY)
         radio.wifi_mode = None
         self.assertEqual(radio.wifi_mode, None)
 
@@ -965,7 +978,7 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         wlink.errctx = network_element_update.ErrorContext()
         self.assertEqual(wlink.connected_time, None)
-        wlink.connected_time = pow(2,31) + 100
+        wlink.connected_time = pow(2, 31) + 100
         self.assertEqual(wlink.connected_time, pow(2, 31) + 100)
         wlink.connected_time = -1
         self.assertEqual(wlink.connected_time, pow(2, 31) + 100)
@@ -1135,3 +1148,238 @@ class TestNetworkElementUpdate(unittest.TestCase):
 
         sector.mode = network_poller_pb2.SectorMode.AP
         self.assertEqual(sector.mode, network_poller_pb2.SectorMode.AP)
+
+        # Module
+        module = ne.modules.add()
+        module.id = 'module1'
+        self.assertEqual(module.id, 'module1')
+
+        poller_hash = sha256().digest()
+        module.poller_hash = poller_hash
+        self.assertEqual(poller_hash, module.poller_hash)
+
+        self.assertEqual(module.name, None)
+        module.name = 'module-name'
+        self.assertEqual(module.name, 'module-name')
+        module.name = None
+        self.assertEqual(module.name, None)
+
+        self.assertEqual(module.manufacturer, None)
+        module.manufacturer = 'module-mfg'
+        self.assertEqual(module.manufacturer, 'module-mfg')
+        module.manufacturer = None
+        self.assertEqual(module.manufacturer, None)
+
+        self.assertEqual(module.model, None)
+        module.model = 'module-mod'
+        self.assertEqual(module.model, 'module-mod')
+        module.model = None
+        self.assertEqual(module.model, None)
+
+        self.assertEqual(module.serial_number, None)
+        module.serial_number = 'module-ser'
+        self.assertEqual(module.serial_number, 'module-ser')
+        module.serial_number = None
+        self.assertEqual(module.serial_number, None)
+
+        self.assertEqual(module.sw_version, None)
+        module.sw_version = '1.2.3'
+        self.assertEqual(module.sw_version, '1.2.3')
+        module.sw_version = None
+        self.assertEqual(module.sw_version, None)
+
+        module.errctx = network_element_update.ErrorContext()
+        self.assertEqual(module.uptime, None)
+        module.uptime = 12345
+        self.assertEqual(module.uptime, 12345)
+        module.uptime = -1
+        self.assertEqual(module.uptime, 12345)
+        module.uptime = pow(2, 63) + 1
+        self.assertEqual(module.uptime, 12345)
+        module.uptime = None
+        self.assertEqual(len(module.errctx.errors), 2)
+        module.errctx = None
+
+        module.errctx = network_element_update.ErrorContext()
+        self.assertEqual(module.cpu, None)
+        module.cpu = 5.5
+        self.assertEqual(round(module.cpu, 4), 5.5)
+        module.cpu = -1
+        self.assertEqual(round(module.cpu, 4), 5.5)
+        module.cpu = 100.1
+        self.assertEqual(round(module.cpu, 4), 5.5)
+        module.cpu = None
+        self.assertEqual(module.cpu, None)
+        self.assertEqual(len(module.errctx.errors), 2)
+        module.errctx = None
+
+        module.errctx = network_element_update.ErrorContext()
+        self.assertEqual(module.mem, None)
+        module.mem = 5.6
+        self.assertEqual(round(module.mem, 4), 5.6)
+        module.mem = -1
+        self.assertEqual(round(module.mem, 4), 5.6)
+        module.mem = 100.1
+        self.assertEqual(round(module.mem, 4), 5.6)
+        module.mem = None
+        self.assertEqual(module.mem, None)
+        self.assertEqual(len(module.errctx.errors), 2)
+        module.errctx = None
+
+        self.assertEqual(module.temperature, None)
+        module.temperature = 31.02
+        self.assertEqual(round(module.temperature, 4), 31.02)
+        module.temperature = -10
+        self.assertEqual(round(module.temperature, 4), -10)
+        module.temperature = None
+        self.assertEqual(module.temperature, None)
+
+        intf2 = module.interfaces.add()
+
+        # Pon
+        pon = intf2.pons.add()
+        pon.id = 'pon1'
+        self.assertEqual(pon.id, 'pon1')
+
+        poller_hash = sha256().digest()
+        pon.poller_hash = poller_hash
+        self.assertEqual(poller_hash, pon.poller_hash)
+
+        pon.pon_technology = network_poller_pb2.PonTechnology.APON
+        self.assertEqual(pon.pon_technology, network_poller_pb2.PonTechnology.APON)
+        pon.pon_technology = None
+        self.assertEqual(pon.pon_technology, None)
+
+        pon.errctx = network_element_update.ErrorContext()
+        self.assertEqual(pon.split_ratio, None)
+        pon.split_ratio = 5
+        self.assertEqual(pon.split_ratio, 5)
+        pon.split_ratio = -1
+        self.assertEqual(pon.split_ratio, 5)
+        pon.split_ratio = pow(2, 31) + 1
+        self.assertEqual(pon.split_ratio, 5)
+        pon.split_ratio = None
+        self.assertEqual(pon.split_ratio, None)
+        self.assertEqual(len(pon.errctx.errors), 2)
+        pon.errctx = None
+
+        pon.errctx = network_element_update.ErrorContext()
+        self.assertEqual(pon.max_distance, None)
+        pon.max_distance = 600
+        self.assertEqual(pon.max_distance, 600)
+        pon.max_distance = -1
+        self.assertEqual(pon.max_distance, 600)
+        pon.max_distance = pow(2, 31) + 1
+        self.assertEqual(pon.max_distance, 600)
+        pon.max_distance = None
+        self.assertEqual(pon.max_distance, None)
+        self.assertEqual(len(pon.errctx.errors), 2)
+        pon.errctx = None
+
+        self.assertEqual(pon.minimum_loss, None)
+        pon.minimum_loss = 5.02
+        self.assertEqual(round(pon.minimum_loss, 4), 5.02)
+        pon.minimum_loss = None
+        self.assertEqual(pon.minimum_loss, None)
+
+        self.assertEqual(pon.maximum_loss, None)
+        pon.maximum_loss = 6.02
+        self.assertEqual(round(pon.maximum_loss, 4), 6.02)
+        pon.maximum_loss = None
+        self.assertEqual(pon.maximum_loss, None)
+
+        pon.errctx = network_element_update.ErrorContext()
+        self.assertEqual(pon.rx_link_rate, None)
+        pon.rx_link_rate = 7449000000
+        self.assertEqual(pon.rx_link_rate, 7449000000)
+        pon.rx_link_rate = -1
+        self.assertEqual(pon.rx_link_rate, 7449000000)
+        pon.rx_link_rate = pow(2, 63) + 1
+        self.assertEqual(pon.rx_link_rate, 7449000000)
+        pon.rx_link_rate = None
+        self.assertEqual(pon.rx_link_rate, None)
+        self.assertEqual(len(pon.errctx.errors), 2)
+        pon.errctx = None
+
+        pon.errctx = network_element_update.ErrorContext()
+        self.assertEqual(pon.tx_link_rate, None)
+        pon.tx_link_rate = 17449000000
+        self.assertEqual(pon.tx_link_rate, 17449000000)
+        pon.tx_link_rate = -1
+        self.assertEqual(pon.tx_link_rate, 17449000000)
+        pon.tx_link_rate = pow(2, 63) + 1
+        self.assertEqual(pon.tx_link_rate, 17449000000)
+        pon.tx_link_rate = None
+        self.assertEqual(pon.tx_link_rate, None)
+        self.assertEqual(len(pon.errctx.errors), 2)
+        pon.errctx = None
+
+        # PonLink
+        ponlink = pon.links.add()
+        poller_hash = sha256().digest()
+        ponlink.poller_hash = poller_hash
+        self.assertEqual(poller_hash, ponlink.poller_hash)
+
+        self.assertEqual(ponlink.rx_power, None)
+        ponlink.rx_power = 90.59
+        self.assertEqual(round(ponlink.rx_power, 4), 90.59)
+        ponlink.rx_power = None
+        self.assertEqual(ponlink.rx_power, None)
+
+        self.assertEqual(ponlink.tx_power, None)
+        ponlink.tx_power = 80.59
+        self.assertEqual(round(ponlink.tx_power, 4), 80.59)
+        ponlink.tx_power = None
+        self.assertEqual(ponlink.tx_power, None)
+
+        ponlink.errctx = network_element_update.ErrorContext()
+        self.assertEqual(ponlink.fiber_length, None)
+        ponlink.fiber_length = 200
+        self.assertEqual(ponlink.fiber_length, 200)
+        ponlink.fiber_length = -1
+        self.assertEqual(ponlink.fiber_length, 200)
+        ponlink.fiber_length = pow(2, 31) + 1
+        self.assertEqual(ponlink.fiber_length, 200)
+        ponlink.fiber_length = None
+        self.assertEqual(ponlink.fiber_length, None)
+        self.assertEqual(len(ponlink.errctx.errors), 2)
+        ponlink.errctx = None
+
+        ponlink.errctx = network_element_update.ErrorContext()
+        self.assertEqual(ponlink.connected_time, None)
+        ponlink.connected_time = 123456
+        self.assertEqual(ponlink.connected_time, 123456)
+        ponlink.connected_time = -1
+        self.assertEqual(ponlink.connected_time, 123456)
+        ponlink.connected_time = pow(2, 63) + 1
+        self.assertEqual(ponlink.connected_time, 123456)
+        ponlink.connected_time = None
+        self.assertEqual(ponlink.connected_time, None)
+        self.assertEqual(len(ponlink.errctx.errors), 2)
+        ponlink.errctx = None
+
+        ponlink.errctx = network_element_update.ErrorContext()
+        self.assertEqual(ponlink.rx_link_rate, None)
+        ponlink.rx_link_rate = 7449000000
+        self.assertEqual(ponlink.rx_link_rate, 7449000000)
+        ponlink.rx_link_rate = -1
+        self.assertEqual(ponlink.rx_link_rate, 7449000000)
+        ponlink.rx_link_rate = pow(2, 63) + 1
+        self.assertEqual(ponlink.rx_link_rate, 7449000000)
+        ponlink.rx_link_rate = None
+        self.assertEqual(ponlink.rx_link_rate, None)
+        self.assertEqual(len(ponlink.errctx.errors), 2)
+        ponlink.errctx = None
+
+        ponlink.errctx = network_element_update.ErrorContext()
+        self.assertEqual(ponlink.tx_link_rate, None)
+        ponlink.tx_link_rate = 17449000000
+        self.assertEqual(ponlink.tx_link_rate, 17449000000)
+        ponlink.tx_link_rate = -1
+        self.assertEqual(ponlink.tx_link_rate, 17449000000)
+        ponlink.tx_link_rate = pow(2, 63) + 1
+        self.assertEqual(ponlink.tx_link_rate, 17449000000)
+        ponlink.tx_link_rate = None
+        self.assertEqual(ponlink.tx_link_rate, None)
+        self.assertEqual(len(ponlink.errctx.errors), 2)
+        ponlink.errctx = None
